﻿Imports TileServer
Imports System.Net
Imports System.Drawing

Public MustInherit Class Google

    Implements IServer

    Protected ImageTypeValue As String
    Protected MaximumZoomValue As Integer
    Protected ServerNameValue As String
    Protected URL(3) As String

   
    Protected Function Tilename(ByVal X As Integer, ByVal Y As Integer, ByVal Zoom As Integer) As String
        Tilename = "t"
        Dim N As Integer
        Dim Limit As Integer

        For N = Zoom To 0 Step -1
            Limit = 2 ^ N
            If X < Limit Then
                If Y < Limit Then
                    Tilename = Tilename & "q"
                Else
                    Tilename = Tilename & "t"
                    Y = Y - Limit
                End If
            Else
                If Y < Limit Then
                    Tilename = Tilename & "r"
                Else
                    Tilename = Tilename & "s"
                    Y = Y - Limit
                End If
                X = X - Limit
            End If
        Next

    End Function

    Public Overridable Function DownloadTile(ByVal X As Integer, ByVal Y As Integer, ByVal Zoom As Integer, ByVal Filename As String) As Boolean Implements TileServer.IServer.DownloadTile
        Dim XY As Integer = 2 ^ (Zoom + 1)
        X = X Mod XY
        Y = Y Mod XY
        Dim Tile As String = Tilename(X, Y, Zoom)
        Dim S As Integer = (X + Y) Mod 4
        Dim imageURL As String = URL(S) & Tile
        Dim index As Integer
        Dim img As Image
        Try
            Dim req As Net.HttpWebRequest = DirectCast(Net.HttpWebRequest.Create(imageURL), Net.HttpWebRequest)
            req.UserAgent = "Mozilla/4.0 (compatible; MSIE 6.0;Windows NT 5.1"
            Dim res As Net.HttpWebResponse = DirectCast(req.GetResponse, Net.HttpWebResponse)
            index = res.ContentType.IndexOf("image")
            If index > -1 Then
                img = Image.FromStream(res.GetResponseStream)
                img.Save(Filename)
                DownloadTile = True
            Else
                DownloadTile = False
            End If
            res.Close()
        Catch ex As Exception
            DownloadTile = False
        End Try

    End Function

    Public ReadOnly Property ImageType() As String Implements TileServer.IServer.ImageType
        Get
            ImageType = ImageTypeValue
        End Get
    End Property
    Public ReadOnly Property MaximumZoom() As Integer Implements TileServer.IServer.MaximumZoom
        Get
            MaximumZoom = MaximumZoomValue
        End Get
    End Property
    Public ReadOnly Property ServerName() As String Implements TileServer.IServer.ServerName
        Get
            ServerName = ServerNameValue
        End Get
    End Property
    Protected Property BaseURL0() As String
        Get
            BaseURL0 = URL(0)
        End Get
        Set(ByVal value As String)
            URL(0) = value
        End Set
    End Property
    Protected Property BaseURL1() As String
        Get
            BaseURL1 = URL(1)
        End Get
        Set(ByVal value As String)
            URL(1) = value
        End Set
    End Property
    Protected Property BaseURL2() As String
        Get
            BaseURL2 = URL(2)
        End Get
        Set(ByVal value As String)
            URL(2) = value
        End Set
    End Property
    Protected Property BaseURL3() As String
        Get
            BaseURL3 = URL(3)
        End Get
        Set(ByVal value As String)
            URL(3) = value
        End Set
    End Property
    Protected Property BaseImageType() As String
        Get
            BaseImageType = ImageTypeValue
        End Get
        Set(ByVal value As String)
            ImageTypeValue = value
        End Set
    End Property
    Protected Property BaseServerName() As String
        Get
            BaseServerName = ServerNameValue
        End Get
        Set(ByVal value As String)
            ServerNameValue = value
        End Set
    End Property
    Protected Property BaseMaximumZoom() As Integer
        Get
            BaseMaximumZoom = MaximumZoomValue
        End Get
        Set(ByVal value As Integer)
            MaximumZoomValue = value
        End Set
    End Property
End Class

Public Class GoogleSatellite : Inherits Google
    Sub New()
        MyBase.New()
        BaseImageType = ".JPG"
        BaseMaximumZoom = 20
        BaseServerName = "GoogleSatellite"
        BaseURL0 = "http://khm0.google.com/kh/v=60&x="
        BaseURL1 = "http://khm1.google.com/kh/v=60&x="
        BaseURL2 = "http://khm2.google.com/kh/v=60&x="
        BaseURL3 = "http://khm3.google.com/kh/v=60&x="

    End Sub

    Public Overrides Function DownloadTile(ByVal X As Integer, ByVal Y As Integer, ByVal Zoom As Integer, ByVal Filename As String) As Boolean
        Dim XY As Integer = 2 ^ (Zoom + 1)
        X = X Mod XY
        Y = Y Mod XY
        Dim Tile As String = CStr(X) & "&y=" & CStr(Y) & "&zoom=" & CStr(16 - Zoom)
        Dim S As Integer = (X + Y) Mod 4
        Dim imageURL As String = URL(S) & Tile
        Dim index As Integer
        Dim img As Image
        Try
            Dim req As Net.HttpWebRequest = DirectCast(Net.HttpWebRequest.Create(imageURL), Net.HttpWebRequest)
            req.UserAgent = "Mozilla/4.0 (compatible; MSIE 6.0;Windows NT 5.1"
            Dim res As Net.HttpWebResponse = DirectCast(req.GetResponse, Net.HttpWebResponse)
            index = res.ContentType.IndexOf("image")
            If index > -1 Then
                img = Image.FromStream(res.GetResponseStream)
                img.Save(Filename)
                DownloadTile = True
            Else
                DownloadTile = False
            End If
            res.Close()
        Catch ex As Exception
            DownloadTile = False
        End Try
    End Function
    
End Class

Public Class GoogleStreetMap : Inherits Google
    Sub New()

        MyBase.New()
        BaseImageType = ".PNG"
        BaseMaximumZoom = 16
        BaseServerName = "GoogleStreetMap"
        BaseURL0 = "http://mt0.google.com/vt/src=api&x="
        BaseURL1 = "http://mt1.google.com/vt/src=api&x="
        BaseURL2 = "http://mt2.google.com/vt/src=api&x="
        BaseURL3 = "http://mt3.google.com/vt/src=api&x="

    End Sub

    Public Overrides Function DownloadTile(ByVal X As Integer, ByVal Y As Integer, ByVal Zoom As Integer, ByVal Filename As String) As Boolean
        Dim XY As Integer = 2 ^ (Zoom + 1)
        X = X Mod XY
        Y = Y Mod XY
        Dim Tile As String = CStr(X) & "&y=" & CStr(Y) & "&zoom=" & CStr(16 - Zoom)
        Dim S As Integer = (X + Y) Mod 4
        Dim imageURL As String = URL(S) & Tile
        Dim index As Integer
        Dim img As Image
        Try
            Dim req As Net.HttpWebRequest = DirectCast(Net.HttpWebRequest.Create(imageURL), Net.HttpWebRequest)
            req.UserAgent = "Mozilla/4.0 (compatible; MSIE 6.0;Windows NT 5.1"
            Dim res As Net.HttpWebResponse = DirectCast(req.GetResponse, Net.HttpWebResponse)
            index = res.ContentType.IndexOf("image")
            If index > -1 Then
                img = Image.FromStream(res.GetResponseStream)
                img.Save(Filename)
                DownloadTile = True
            Else
                DownloadTile = False
            End If
            res.Close()
        Catch ex As Exception
            DownloadTile = False
        End Try
    End Function

End Class

